/*
 * GStreamer gst-template:
 * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
 * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
 * 
 * Original dvdreadsrc:
 * Copyright (C) 1999 Erik Walthinsen <omega@cse.ogi.edu>
 * Copyright (C) 2001 Billy Biggs <vektor@dumbterm.net>.
 * Copyright (C) 2006 Tim-Philipp Müller <tim centricular net>
 *
 * New dvdsrc, based on gst-template and dvdreadsrc:
 * Copyright (C) 2006 Jason Gerard DeRose <jderose@jasonderose.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GST_DVDSRC_H__
#define __GST_DVDSRC_H__

#include <gst/gst.h>
#include <gst/base/gstpushsrc.h>
#include <glib/gstdio.h>
#include <inttypes.h>
#include <dvdread/dvd_reader.h>
#include <dvdread/ifo_types.h>
#include <dvdread/ifo_read.h>
#include <dvdread/nav_read.h>
#include <dvdread/nav_print.h>



G_BEGIN_DECLS

/* #defines don't like whitespacey bits */
#define GST_TYPE_DVDSRC \
  (gst_dvdsrc_get_type())
#define GST_DVDSRC(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_DVDSRC,GstDvdSrc))
#define GST_DVDSRC_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DVDSRC,GstDvdSrcClass))
#define GST_IS_PLUGIN_TEMPLATE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DVDSRC))
#define GST_IS_PLUGIN_TEMPLATE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_DVDSRC))

typedef struct _GstDvdSrc      GstDvdSrc;
typedef struct _GstDvdSrcClass GstDvdSrcClass;

struct _GstDvdSrc
{
  GstPushSrc        element;

  /* Properties */
  gboolean          prop_audit;
  gint              prop_title;
  gint              prop_chapter;
  gint              prop_angle;
  gchar           * prop_device;
  
  gchar           * last_uri;
  
  /* Current sector */
  gint              sector;
  
  /* DVD related */
  gint              dvd_titles;
  dvd_reader_t    * dvd_reader;
  ifo_handle_t    * dvd_vmg;
  
  /* Title related */
  gint              title_i;
  gint				  	  title_set;
  gint			  		  title_ttn;
  gint		  		    title_chapters;
  gint              title_angles;
  gint              title_seconds;
  gint              title_firstSector;
  gint              title_lastSector;
  ifo_handle_t    * title_vts;
  dvd_file_t      * title_file;
  vts_tmap_t      * title_tmap;
  ttu_t           * title_ttu;
  
  /* Chapter related */
  gint              chapter_i;
  gint              chapter_pgcSeconds;
  gint              chapter_firstCell;
  gint              chapter_lastCell;
  gint              chapter_firstSector;
  gint              chapter_lastSector;
  pgc_t           * chapter_pgc;
  
  /* Cell related */
  gint              cell_i;
  gint              cell_firstSector;
  gint              cell_lastSector;
  gint              cell_seconds;
  cell_playback_t * cell_playback;
  
  /* Events */
  GstEvent        * pending_lang_event;
  GstEvent        * pending_clut_event;
  
};

struct _GstDvdSrcClass 
{
  GstPushSrcClass parent_class;
};

GType gst_dvdsrc_get_type (void);

G_END_DECLS


#endif /* __GST_DVDSRC_H__ */
