# This needs to be cleaned up, but this test compares the bitstream produced
# by my dvdsrc to the original dvdreadsrc (from cvs).
#
# For a given DVD title, it compares:
#  * the number of buffers output
#  * the sha1 hash of each buffer.data
#  * the size of each buffer
#  * the timestamp of each buffer
#
# As of this writting (Bazzar revno: 38), this test has passed on the several
# DVDs I've fed it.  It will fail if using gst-plugins-ugly 0.10.4 rather
# than CVS, as Tim patched CVS such that the Nav-block plus data packet is
# sent in each buffer (which is also what I do in dvdsrc, following his lead).


import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst

import sha


list1 = []
list2 = []

def get_time(ts):
	if ts == gst.CLOCK_TIME_NONE:
		return 'None'
	
	sec = ts / gst.SECOND
	return '%d:%02d' % (sec/60, sec%60)

def do_start():
	print 'do_start'
	Tester('dvdsrc', list1, do_next)
	
def do_next():
	print 'do_next'
	Tester('dvdreadsrc', list2, do_compare)
	
def do_compare():
	print 'do_compare'
	for i in xrange(min(len(list1), len(list2))):
		print 'dvdsrc:', list1[i]
		print 'dvdreadsrc:', list2[i]
		for j in xrange(3):
			if list1[i][j] != list2[i][j]:
				print 'FAIL!'
				return
		print 'OKAY'
	print 'dvdsrc:', len(list1)
	print 'dvdreadsrc:', len(list2)
	if len(list1) == len(list2):
		print 'OKAY: number of packets'
	else:
		print 'FAIL: number of packets'


class MySink(gst.Element):

	_sinkpadtemplate = gst.PadTemplate ("sinkpadtemplate",
										gst.PAD_SINK,
										gst.PAD_ALWAYS,
										gst.caps_new_any())

	def __init__(self, src_list):
		gst.Element.__init__(self)
		gst.info('creating sinkpad')
		self.sinkpad = gst.Pad(self._sinkpadtemplate, "sink")
		gst.info('adding sinkpad to self')
		self.add_pad(self.sinkpad)

		gst.info('setting chain/event functions')
		self.sinkpad.set_chain_function(self.chainfunc)
		self.sinkpad.set_event_function(self.eventfunc)
		
		self.src_list = src_list
		
	def chainfunc(self, pad, buf):
		s = sha.new(buf.data)
		if buf.timestamp != gst.CLOCK_TIME_NONE:
			print get_time(buf.timestamp)
		self.src_list.append( (s.hexdigest(), buf.size, get_time(buf.timestamp)))
		return gst.FLOW_OK

	def eventfunc(self, pad, event):
		print "%s event:%r" % (pad, event.type)
		if event.type == gst.EVENT_EOS:
			print 'sending eos'
			self.post_message(gst.message_new_eos(self))
		return True

gobject.type_register(MySink)



class Tester:
	def __init__(self, src, src_list, callback):
		self.callback = callback
		
		# The pipeline
		self.pipeline = gst.Pipeline()
		
		# Create bus and connect several handlers
		self.bus = self.pipeline.get_bus()
		self.bus.add_signal_watch()
		self.bus.connect('message::eos', self.on_eos)
		self.bus.connect('message::error', self.on_error)

		# Create elements
		self.src = gst.element_factory_make(src)
		self.sink = MySink(src_list)
		#self.sink = gst.element_factory_make('fakesink')

		#self.src.set_property('device', '/home/jderose/SEALAB_2021_DISC_2_NA/')
		#self.src.set_property('title', 7)
		
		# Add elements to pipeline
		self.pipeline.add(self.src, self.sink)
		
		# Link elements 
		self.src.link(self.sink)

		# The MainLoop
		self.mainloop = gobject.MainLoop()
		
		# And off we go!
		self.pipeline.set_state(gst.STATE_PLAYING)
		#gobject.timeout_add(20000, self.on_timeout)
		self.mainloop.run()
		

	def on_timeout(self):
		self.pipeline.set_state(gst.STATE_NULL)
		self.pipeline.get_state()
		self.mainloop.quit()
		self.callback()
		return False # Done repeat timeout call			
		
			
	def on_message(self, bus, msg):
	  print '\n\non_message', dir(msg)
			
	def on_eos(self, bus, msg):
		print 'on_eos'
		self.pipeline.set_state(gst.STATE_NULL)
		self.pipeline.get_state()
		self.mainloop.quit()
		self.callback()
		
		
			
	def on_error(self, bus, msg):
		error = msg.parse_error()
		print 'on_error:', error
		self.mainloop.quit()
		
	


		
do_start()
